@@ -29,6 +29,7 @@ from gi.repository import GIRepository  # type: ignore
 
 from . import typelib, xml_reader
 from .errors import CompileError, CompilerBugError
+from .lsp_utils import CodeAction
 
 _namespace_cache: T.Dict[str, "Namespace"] = {}
 _xml_cache = {}
@@ -65,6 +66,32 @@ def get_namespace(namespace: str, version: str) -> "Namespace":
     return _namespace_cache[filename]
 
 
+_available_namespaces: list[tuple[str, str]] = []
+
+
+def get_available_namespaces() -> T.List[T.Tuple[str, str]]:
+    if len(_available_namespaces):
+        return _available_namespaces
+
+    search_paths: list[str] = [
+        *GIRepository.Repository.get_search_path(),
+        *_user_search_paths,
+    ]
+
+    for search_path in search_paths:
+        try:
+            filenames = os.listdir(search_path)
+        except FileNotFoundError:
+            continue
+
+        for filename in filenames:
+            if filename.endswith(".typelib"):
+                namespace, version = filename.removesuffix(".typelib").rsplit("-", 1)
+                _available_namespaces.append((namespace, version))
+
+    return _available_namespaces
+
+
 def get_xml(namespace: str, version: str):
     search_paths = []
 
@@ -136,6 +163,14 @@ class GirType:
     def incomplete(self) -> bool:
         return False
 
+    @property
+    def deprecated(self) -> bool:
+        return False
+
+    @property
+    def deprecated_doc(self) -> T.Optional[str]:
+        return None
+
 
 class ExternType(GirType):
     def __init__(self, name: str) -> None:
@@ -165,6 +200,10 @@ class ArrayType(GirType):
     def assignable_to(self, other: GirType) -> bool:
         return isinstance(other, ArrayType) and self._inner.assignable_to(other._inner)
 
+    @property
+    def inner(self) -> GirType:
+        return self._inner
+
     @property
     def name(self) -> str:
         return self._inner.name + "[]"
@@ -297,6 +336,17 @@ class GirNode:
     def available_in(self) -> str:
         return self.xml.get("version")
 
+    @cached_property
+    def detail(self) -> T.Optional[str]:
+        try:
+            el = self.xml.get_elements("doc")
+            if len(el) == 1:
+                return el[0].cdata.strip().partition("\n")[0]
+            else:
+                return None
+        except:
+            return None
+
     @cached_property
     def doc(self) -> T.Optional[str]:
         sections = []
@@ -330,6 +380,13 @@ class GirNode:
     def type(self) -> GirType:
         raise NotImplementedError()
 
+    @property
+    def deprecated_doc(self) -> T.Optional[str]:
+        try:
+            return self.xml.get_elements("doc-deprecated")[0].cdata.strip()
+        except:
+            return None
+
 
 class Property(GirNode):
     xml_tag = "property"
@@ -365,6 +422,10 @@ class Property(GirNode):
         else:
             return None
 
+    @property
+    def deprecated(self) -> bool:
+        return self.tl.PROP_DEPRECATED == 1
+
 
 class Argument(GirNode):
     def __init__(self, container: GirNode, tl: typelib.Typelib) -> None:
@@ -427,6 +488,10 @@ class Signal(GirNode):
         else:
             return None
 
+    @property
+    def deprecated(self) -> bool:
+        return self.tl.SIGNAL_DEPRECATED == 1
+
 
 class Interface(GirNode, GirType):
     xml_tag = "interface"
@@ -484,10 +549,14 @@ class Interface(GirNode, GirType):
     @property
     def online_docs(self) -> T.Optional[str]:
         if ns := self.get_containing(Namespace).online_docs:
-            return f"{ns}interface.{self.name}.html"
+            return f"{ns}iface.{self.name}.html"
         else:
             return None
 
+    @property
+    def deprecated(self) -> bool:
+        return self.tl.INTERFACE_DEPRECATED == 1
+
 
 class Class(GirNode, GirType):
     xml_tag = "class"
@@ -609,6 +678,10 @@ class Class(GirNode, GirType):
         else:
             return None
 
+    @property
+    def deprecated(self) -> bool:
+        return self.tl.OBJ_DEPRECATED == 1
+
 
 class TemplateType(GirType):
     def __init__(self, name: str, parent: T.Optional[GirType]):
@@ -648,7 +721,7 @@ class TemplateType(GirType):
             # we don't know the template type's interfaces, assume yes
             return True
         elif self.parent is None or isinstance(self.parent, ExternType):
-            return isinstance(other, Class)
+            return isinstance(other, Class) or isinstance(other, ExternType)
         else:
             return self.parent.assignable_to(other)
 
@@ -722,6 +795,10 @@ class Enumeration(GirNode, GirType):
         else:
             return None
 
+    @property
+    def deprecated(self) -> bool:
+        return self.tl.ENUM_DEPRECATED == 1
+
 
 class Boxed(GirNode, GirType):
     xml_tag = "glib:boxed"
@@ -743,6 +820,10 @@ class Boxed(GirNode, GirType):
         else:
             return None
 
+    @property
+    def deprecated(self) -> bool:
+        return self.tl.STRUCT_DEPRECATED == 1
+
 
 class Bitfield(Enumeration):
     xml_tag = "bitfield"
@@ -823,8 +904,16 @@ class Namespace(GirNode):
 
     def get_type_by_cname(self, cname: str) -> T.Optional[GirType]:
         """Gets a type from this namespace by its C name."""
+        for basic in _BASIC_TYPES.values():
+            if basic.glib_type_name == cname:
+                return basic()
+
         for item in self.entries.values():
-            if hasattr(item, "cname") and item.cname == cname:
+            if (
+                hasattr(item, "cname")
+                and item.cname is not None
+                and item.cname == cname
+            ):
                 return item
         return None
 
@@ -888,8 +977,8 @@ class Repository(GirNode):
             return self.lookup_namespace(ns).get_type(dir_entry.DIR_ENTRY_NAME)
 
     def _resolve_type_id(self, type_id: int) -> GirType:
-        if type_id & 0xFFFFFF == 0:
-            type_id = (type_id >> 27) & 0x1F
+        if type_id & (0xFFFFFF if sys.byteorder == "little" else 0xFFFFFF00) == 0:
+            type_id = ((type_id >> 27) if sys.byteorder == "little" else type_id) & 0x1F
             # simple type
             if type_id == typelib.TYPE_BOOLEAN:
                 return BoolType()
@@ -972,9 +1061,11 @@ class GirContext:
         ns = ns or "Gtk"
 
         if ns not in self.namespaces and ns not in self.not_found_namespaces:
+            all_available = list(set(ns for ns, _version in get_available_namespaces()))
+
             raise CompileError(
                 f"Namespace {ns} was not imported",
-                did_you_mean=(ns, self.namespaces.keys()),
+                did_you_mean=(ns, all_available),
             )
 
     def validate_type(self, name: str, ns: str) -> None:
